{{% set title = "Rules with " ~ reference_title ~ " Reference in Guide to the Secure Configuration of " ~ product_full_name -%}}

{{% extends "tables_template_common.html" %}}

{{% block table %}}
<table>
  <thead>
    <th>Mapping</th>
    <th>Rule Title</th>
    <th>Description</th>
    <th>Rationale</th>
  </thead>
  <tbody>
  {{% for ref in sorted_refs -%}}
    {{% set rules = rules_by_shortref[ref] -%}}
    {{% for rule in rules -%}}
    <tr>
      <td>{{{ "<br/>".join(rule.relevant_refs) }}}</td>
      <td>{{{ rule.title }}}</td>
      <td xml:lang="en-US">
        {{{ rule.description }}}
      </td>
      <td xml:lang="en-US">
        {{{ rule.rationale }}}
      </td>
    </tr>
    {{% endfor -%}}
  {{% endfor -%}}
  </tbody>
</table>
{{% endblock %}}
