<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8"/>
  <title>Rendered Policies</title>
<style>
</style>
</head>
<body>

<h1>Rendered Policies</h1>

{{% for product in products|sort(attribute="name") %}}
<h4>{{{ product.name }}}</h4>
  <ul>
    {{% for policy_id in product.policy_ids %}}
    <li>
      <a href="{{{ product.id }}}/{{{ policy_id }}}.html">{{{ policy_id }}}: {{{ policy_names[policy_id] }}}</a>
    </li>
    {{% endfor %}}
  </ul>
{{% endfor %}}

</body>
</html>
