<!DOCTYPE html>
<html lang="en">
<head>
  {{% block head %}}
  <meta charset="UTF-8"/>
  <title>{{{ title }}}</title>
  {{% endblock %}}
<style>
</style>
</head>
<body>

{{% block body %}}
<h1>{{{ title }}}</h1>
{{% endblock %}}

</body>
</html>
