#!/bin/bash
# profiles = xccdf_org.ssgproject.content_profile_cis

# CIS doesn't have an explicit content required, but doesn't expect technical information about the OS.
echo "Sytem name \s version \s " > /etc/issue.net
