# platform = multi_platform_rhel,multi_platform_ol,multi_platform_almalinux

{{{ bash_create_audit_remediation_unsuccessful_file_modification_detailed("/etc/audit/rules.d/30-ospp-v42-remediation.rules") }}}
