#!/bin/bash
# platform = Oracle Linux 8,multi_platform_fedora,multi_platform_rhel,multi_platform_rhv,multi_platform_sle

ln -sf /lib/systemd/system/graphical.target /etc/systemd/system/default.target
