# platform = multi_platform_rhel,multi_platform_fedora,multi_platform_ol,multi_platform_rhv,multi_platform_sle,multi_platform_ubuntu,multi_platform_almalinux

{{{ bash_pam_faillock_enable() }}}
{{{ bash_pam_faillock_parameter_value("audit", authfail=False)}}}
