# platform = multi_platform_all
# reboot = false
# strategy = restrict
# complexity = low
# disruption = low

{{{ bash_sudo_remove_config("NOPASSWD", "NOPASSWD[\s]*\:") }}}

{{{ bash_sudo_remove_config("!authenticate", "\!authenticate") }}}
