#!/bin/bash

echo """
This script has been deprecated in favor of build_ds_container.py. The
build_ds_container.py script supports all the same parameters as this script,
but please see ./build_ds_container.py -h for specific details and usage.

This script will be removed in a future patch.
"""

exit 0
